package main

import (
	"dwelling-upload/internal/http"
	"dwelling-upload/pkg/logging"
	"dwelling-upload/pkg/utils"
	"dwelling-upload/pkg/watcher"
	"flag"
	"fmt"
	"log"
	"net/netip"
	"os"
	"os/signal"
	"path"
	"strings"
	"syscall"
)

var configPath *string = flag.String("conf", "config.yaml", "path to configuration file")
var listenAddress *string = flag.String("listen", "/var/run/dwelling-upload/sock", "listen address (ip:port|unix_path)")
var uploadDir *string = flag.String("upload-dir", "/srv/uploads", "path to a directory where uploaded files are stored")
var keepFileForHours *int = flag.Int("keep-for", 36, "keep files for this much hours")
var limitStorage *int64 = flag.Int64("storage", 102400, "storage size in MiB for uploads")
var limitFileSize *int64 = flag.Int64("file-size", 128, "max. size in MiB for uploaded files")
var showVersion *bool = flag.Bool("v", false, "show version")

var version string

func main() {
	flag.Parse()
	log.SetFlags(log.Llongfile)

	if *showVersion {
		fmt.Println("dwelling-upload Ver. ", version, "\nCopyright (c) 2022,2023 Alexander \"Arav\" Andreev <me@arav.su>")
		return
	}

	var network string
	if !strings.ContainsRune(*listenAddress, ':') {
		network = "unix"
		defer os.Remove(*listenAddress)
	} else {
		ap, err := netip.ParseAddrPort(*listenAddress)
		if err != nil {
			log.Fatalln(err)
		} else if !ap.IsValid() {
			log.Fatalln(*listenAddress, "is not valid")
		}

		if ap.Addr().Is4() {
			network = "tcp4"
		} else if ap.Addr().Is6() {
			network = "tcp6"
		}
	}

	hashSalt, err := os.ReadFile(path.Join(os.Getenv("CREDENTIALS_DIRECTORY"), "salt"))
	if err != nil {
		log.Fatalln("failed to read hash salt file:", err)
	}

	logFilePath := path.Join(os.Getenv("LOGS_DIRECTORY"), "file.log")
	logFile, err := logging.New(logFilePath)
	if err != nil {
		log.Fatalln("file logger:", err)
	}
	defer logFile.Close()

	watcha, err := watcher.NewInotifyWatcher()
	if err != nil {
		log.Fatalln(err)
	}
	defer watcha.Close()

	if err := watcha.AddWatch(*uploadDir, watcher.CrDelMask); err != nil {
		log.Fatalln(err)
	}

	uploadDirNotify := make(chan uint32)
	uploadDirSize, err := utils.DirectorySize(*uploadDir)
	if err != nil {
		log.Fatalf("failed to get initial size of %s: %s", *uploadDir, err)
	}

	watcha.WatchForMask(uploadDirNotify, watcher.CrDelMask)

	hand := http.NewUploadHandlers(logFile, *uploadDir, &uploadDirSize, string(hashSalt),
		*keepFileForHours, *limitStorage, *limitFileSize)
	srv := http.NewHttpServer()

	srv.SetNotFoundHandler(http.NotFound)

	srv.ServeStatic("/assets/*filepath", hand.AssetsFS())
	srv.GET("/robots.txt", http.RobotsTxt)
	srv.GET("/favicon.svg", http.Favicon)

	srv.GET("/", hand.Index)
	srv.POST("/", hand.Upload)
	srv.POST("/delete", hand.Delete)
	srv.GET("/f/:hash/:name", hand.Download)
	srv.DELETE("/:hash", hand.Delete)

	if err := srv.Start(network, *listenAddress); err != nil {
		log.Fatalln("failed to start a server:", err)
	}

	doneSignal := make(chan os.Signal, 1)
	signal.Notify(doneSignal, os.Interrupt, syscall.SIGINT, syscall.SIGTERM)
	logReopenSignal := make(chan os.Signal, 1)
	signal.Notify(logReopenSignal, syscall.SIGHUP)

	go func() {
		for {
			select {
			case <-logReopenSignal:
				logFile.Reopen(logFilePath)
			case <-uploadDirNotify:
				uploadDirSize, err = utils.DirectorySize(*uploadDir)
				if err != nil {
					log.Println("failed to get uploads directory size:", err)
				}
			}
		}
	}()

	<-doneSignal

	if err := srv.Stop(); err != nil {
		log.Fatalln("failed to properly shutdown a server:", err)
	}
}
